//============================================================================
//
//                              Program/Module
//                               from CLAVIS
//                C++ library for verified numerical integration
//                           of singular functions
//                     Copyright (c) 2000 Stefan Wedner
//
//          This program/module is free software for non-commercial use. 
//
// 
//============================================================================
//----------------------------------------------------------------------------
//
// File:    f_ari_test.C
// Purpose: testprogram (f_ari.C)
//
//============================================================================


#include<iostream>
#include"f_ari.h"

typedef  operand& (*o_function)  (operand& Left); 
typedef  operand& (*o2_function) (operand& Left, const real& Right); 
typedef  operand& (*o3_function) (operand& Left, int Right);


// --------------------------------------------------------------------------- 
// Identitaetstest: g = f f^(-1) = id 
// Argumente: Polynome vom Grad grad_s (in s) bzw. Grad grad_t (in t)
// Verglichen werden die Taylorkoeffizienten der Argumente h und g(h)
// Auswertestelle ist s_0 = (s,t) 
// ---------------------------------------------------------------------------

void id_test(o_function f, o_function  f_inv, operand& argu, interval s, 
             int grad_s, interval t =  _interval(0,0), int grad_t = 0){

   integrand g(3,3), h(3,3); operand r;
   g =  f(f_inv(argu)); 
   h =  argu;
   cout << g << endl;
   for(int i = 0; i<=grad_s; i++)
     for(int j = 0; j <= grad_t; j++){
        cout << " TK " << "(" << i << "," << j << ") :" 
             <<  g.taylor(i,j,s,t) << " : " <<  h.taylor(i,j,s,t) << endl;
     }
   
}

// Parametrisieren von id_test2 scheitert, da zB. Typ von pow nicht korrekt 
// ermittelt werden kann. 

void id_test2(o2_function f, o2_function  f_inv, operand& argu, interval s, 
              int grad_s, interval t =  _interval(0,0), int grad_t = 0){

   integrand g(3,3), h(3,3); operand r;
   g =  f(f_inv(argu,0.25), 4); 
   h =  argu;
   cout << g << endl;
   for(int i = 0; i<=grad_s; i++)
     for(int j = 0; j <= grad_t; j++){
        cout << " TK " << "(" << i << "," << j << ") :" 
             <<  g.taylor(i,j,s,t) << " : " <<  h.taylor(i,j,s,t) << endl;
     }
   
}


void id_test3(o3_function f, o2_function  f_inv, operand& argu, interval s, 
              int grad_s, interval t =  _interval(0,0), int grad_t = 0){

   integrand g(3,3), h(3,3); operand r;
   g =  f(f_inv(argu,0.25), 4); 
   h =  argu;
   cout << g << endl;
   for(int i = 0; i<=grad_s; i++)
     for(int j = 0; j <= grad_t; j++){
        cout << " TK " << "(" << i << "," << j << ") :" 
             <<  g.taylor(i,j,s,t) << " : " <<  h.taylor(i,j,s,t) << endl;
     }
   
}

void  test_inverse(o_function f, o_function  f_inv, interval a, interval b){
 
 operand q0, q1, q2, q3, r0, r1, r2, r3, p0, p1 ,p2 ,p3 ,p4 ,p5, 
         t(operand::Var_2), s;

 // operand::set_detail();
 r0 = 1;
 r1 = s;
 r2 = s*s;
 r3 = s*s*s;
 q0 = 1;
 q1 = t;
 q2 = t*t;
 q3 = t*t*t;
 p0 = q0 * r3;
 p1 = q3 * r0;
 p2 = q1 * r3; 
 p3 = q3 * r1;
 p4 = q2 * r2;
 id_test(f,  f_inv, r0,  a, 2); 
 id_test(f, f_inv, r1,  a, 2);
 id_test(f, f_inv, r2,  a, 2);
 id_test(f, f_inv, r3,  a, 2);
 id_test(f, f_inv, q0,  a, 0, b, 2); 
 id_test(f, f_inv, q1,  a, 0, b, 2);
 id_test(f, f_inv, q2,  a, 0, b, 2);
 id_test(f, f_inv, q3,  a, 0, b, 2);
 id_test(f, f_inv, p0 , a, 2, b, 2); 
 id_test(f, f_inv, p1 , a, 2, b, 2);
 id_test(f, f_inv, p2 , a, 2, b, 2); 
 id_test(f, f_inv, p3 , a, 2, b, 2);
 id_test(f, f_inv, p4 , a, 2, b, 2); 
 
}


void  test_pow(interval a, interval b){
 
 operand q0, q1, q2, q3, r0, r1, r2, r3, p0, p1 ,p2 ,p3 ,p4 ,p5, 
         t(operand::Var_2), s;

 // operand::set_detail();
 r0 = 1;
 r1 = s;
 r2 = s*s;
 r3 = s*s*s;
 q0 = 1;
 q1 = t;
 q2 = t*t;
 q3 = t*t*t;
 p0 = q0 * r3;
 p1 = q3 * r0;
 p2 = q1 * r3; 
 p3 = q3 * r1;
 p4 = q2 * r2;
 id_test2(pow, pow, r0,  a, 2); 
 id_test2(pow, pow, r1,  a, 2);
 id_test2(pow, pow, r2,  a, 2);
 id_test2(pow, pow, r3,  a, 2);
 id_test2(pow, pow, q0,  a, 0, b, 2); 
 id_test2(pow, pow, q1,  a, 0, b, 2);
 id_test2(pow, pow, q2,  a, 0, b, 2);
 id_test2(pow, pow, q3,  a, 0, b, 2);
 id_test2(pow, pow, p0 , a, 2, b, 2); 
 id_test2(pow, pow, p1 , a, 2, b, 2);
 id_test2(pow, pow, p2 , a, 2, b, 2); 
 id_test2(pow, pow, p3 , a, 2, b, 2);
 id_test2(pow, pow, p4 , a, 2, b, 2); 
 id_test3(power, pow, r0,  a, 2); 
 id_test3(power, pow, r1,  a, 2);
 id_test3(power, pow, r2,  a, 2);
 id_test3(power, pow, r3,  a, 2);
 id_test3(power, pow, q0,  a, 0, b, 2); 
 id_test3(power, pow, q1,  a, 0, b, 2);
 id_test3(power, pow, q2,  a, 0, b, 2);
 id_test3(power, pow, q3,  a, 0, b, 2);
 id_test3(power, pow, p0 , a, 2, b, 2); 
 id_test3(power, pow, p1 , a, 2, b, 2);
 id_test3(power, pow, p2 , a, 2, b, 2); 
 id_test3(power, pow, p3 , a, 2, b, 2);
 id_test3(power, pow, p4 , a, 2, b, 2); 
}


						     
int main(){

// Einfacher Gebrauch von f_ari
// Berechnung des Tk (2,1) von sinh(x*y) an der Stelle (x0,y0)=(1,2) 

   operand s,t(operand::Var_2);
   integrand f(16,16);    // Maximal TK(16,16) berechenbar 
                          // nachtraeglich mit setdim aenderbar!
   f = sinh(s*t);
   cout << f.taylor(2,1, interval(1, 1), interval(2,2)) << endl;
   cout << f.value(interval(1, 1)) << endl;


//  Abschnitt 1: Test der Elementarfunktionen:
  
test_inverse(exp,ln, interval(2,2), interval(4,4));
// test_inverse(ln,exp, interval(2,2), interval(4,4));
// test_inverse(sqrt,sqr, interval(2,2), interval(4,4));
// test_inverse(sqr,sqrt, interval(2,2), interval(4,4));
// test_inverse(tan, atan,interval(2,2), interval(4,4) );
// test_inverse(asin, sin, interval(0.25,0.25), interval(0.4,0.4));
// test_inverse(acos, cos, interval(0.25,0.25), interval(0.4,0.4));
// test_inverse(atanh, tanh,interval(0.5,0.5), interval(0.4,0.4) );
// test_inverse(asinh, sinh, interval(0.25,0.25), interval(0.4,0.4));
// test_inverse(acosh, cosh, interval(0.25,0.25), interval(0.4,0.4));
// test_pow(interval(0.25,0.25), interval(0.4,0.4));

// Abschnitt 2 Test: Operatoren *, / , =

//  operand r, s, t(operand::Var_2);
//   r = t *s *s;
//   integrand f1(3,3), f2(2,2), f3(4,4), f4(4,4);
//   f1 = pow(r, 0.25);
//   cout << f1 << endl;
//   f2 = pow(s*t*t, 0.25);
//   cout << f2 << endl;
//   f3 = f1 * f1/ r;
//   cout << f3 << endl << endl << " value, diff " << endl; 
//   cout <<  f1.taylor(3, 3, _interval(0.25,0.25), _interval(0.4,0.4)) << endl;
//   cout <<  f1.taylor(0, 0) << endl;
//   cout <<  f1.value(_interval(0.25,0.25), _interval(0.4,0.4)) << endl;
//   cout <<  f2.taylor(2, 2, _interval(0.25,0.25), _interval(0.4,0.4)) << endl; 
//   cout <<  f2.taylor(0, 0) << endl;
//   cout <<  f2.value(_interval(0.25,0.25), _interval(0.4,0.4)) << endl; 
//   cout <<  f3.taylor(4, 4, _interval(0.25,0.25), _interval(0.4,0.4)) << endl;
//   cout <<  f3.taylor(4, 4) << endl;
//   cout <<  f3.value(_interval(0.25,0.25), _interval(0.4,0.4)) << endl;  

//   f4 = f2.transpose();
//   cout <<  endl << f4 << endl;
//   cout <<  f4.taylor(3, 3, _interval(0.25,0.25), _interval(0.4,0.4)) << endl;
//   cout <<  f4.taylor(0, 0) << endl;
//   cout <<  f4.value(_interval(0.25,0.25), _interval(0.4,0.4)) << endl;
#ifdef  MEMORY_CHECK
 cout << " Speichertest :    ";
 if (!testdest()) cout  << "OKAY " << endl;
 else cout << " ERROR " << testdest() << endl;
#endif
 return 0;
}
